----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 23.05.2022 13:41:26
-- Design Name: 
-- Module Name: FSM_Onehot - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity FSM_Onehot is 
 generic (N: integer := 3); 
 port (
    rst          : in std_logic;
    clk          : in std_logic;
    play         : in std_logic;
    state_out    : out std_logic_vector(N-1 downto 0);
    
    error_inject : in std_logic;
    error_data   : in std_logic_vector(1 downto 0);
    alarm        : out std_logic    
);
end FSM_Onehot;

architecture Behavioral of FSM_Onehot is

constant s0 : std_logic_vector(N-1 downto 0) := "00000001";
constant s1 : std_logic_vector(N-1 downto 0) := "00000010";
constant s2 : std_logic_vector(N-1 downto 0) := "00000100";
constant s3 : std_logic_vector(N-1 downto 0) := "00001000";
constant s4 : std_logic_vector(N-1 downto 0) := "00010000";
constant s5 : std_logic_vector(N-1 downto 0) := "00100000";
constant s6 : std_logic_vector(N-1 downto 0) := "01000000";
constant s7 : std_logic_vector(N-1 downto 0) := "10000000"; 

signal current_state : std_logic_vector(N-1 downto 0); 
signal next_state    : std_logic_vector(N-1 downto 0); 

begin

-- State sequence: s0-s1-s2-s3-s4-s5-s6-s7-s0-...
next_state(N-1 downto 1) <= current_state(N-2 downto 0);
next_state(0) <= current_state(N-1);
              
process(rst, clk)
begin 
  if rst = '1' then 
    current_state <= s0;

  elsif rising_edge(clk) then

    if play  = '1' then 
      current_state <= next_state;
    elsif error_inject = '1' then
      current_state(1 downto 0) <= current_state(1 downto 0) XOR error_data;
    end if;

  end if;
end process;

state_out <= current_state;

process(current_state)
variable count1s : integer range 0 to N;
begin
count1s := 0;
  for i in 0 to N-1 loop
    if current_state(i) = '1' then 
      count1s := count1s + 1;
    end if;
  end loop;
  
  if count1s = 1 then
    alarm <= '0';
  else 
    alarm <= '1';
  end if;
end process;
                
end Behavioral;
